@@ -29,10 +29,12 @@ import Text.HTML.TagSoup
 import Text.Pandoc.Class.PandocMonad (PandocMonad (..), fetchItem,
                                       getInputFiles, report, setInputFiles)
 import Text.Pandoc.Logging
+import Text.Pandoc.Error (PandocError(..))
 import Text.Pandoc.MIME (MimeType)
-import Text.Pandoc.Shared (isURI, renderTags', trim)
+import Text.Pandoc.Shared (isURI, renderTags', trim, tshow)
 import Text.Pandoc.UTF8 (toString, toText, fromText)
 import Text.Parsec (ParsecT, runParserT)
+import Control.Monad.Except (throwError, catchError)
 import qualified Text.Parsec as P
 
 isOk :: Char -> Bool
@@ -60,7 +62,7 @@ convertTags :: PandocMonad m => [Tag T.Text] -> m [Tag T.Text]
 convertTags [] = return []
 convertTags (t@TagOpen{}:ts)
   | fromAttrib "data-external" t == "1" = (t:) <$> convertTags ts
-convertTags (t@(TagOpen "script" as):TagClose "script":ts) =
+convertTags (t@(TagOpen "script" as):tc@(TagClose "script"):ts) =
   case fromAttrib "src" t of
        ""  -> (t:) <$> convertTags ts
        src -> do
@@ -68,10 +70,10 @@ convertTags (t@(TagOpen "script" as):TagClose "script":ts) =
            res <- getData typeAttr src
            rest <- convertTags ts
            case res of
-                Left dataUri -> return $ TagOpen "script"
+                AlreadyDataURI dataUri -> return $ TagOpen "script"
                      (("src",dataUri) : [(x,y) | (x,y) <- as, x /= "src"]) :
                      TagClose "script" : rest
-                Right (mime, bs)
+                Fetched (mime, bs)
                   | ("text/javascript" `T.isPrefixOf` mime ||
                      "application/javascript" `T.isPrefixOf` mime ||
                      "application/x-javascript" `T.isPrefixOf` mime) &&
@@ -86,18 +88,19 @@ convertTags (t@(TagOpen "script" as):TagClose "script":ts) =
                          (("src",makeDataURI (mime, bs)) :
                           [(x,y) | (x,y) <- as, x /= "src"]) :
                         TagClose "script" : rest
+                CouldNotFetch _ -> return $ t:tc:rest
 convertTags (t@(TagOpen "link" as):ts) =
   case fromAttrib "href" t of
        ""  -> (t:) <$> convertTags ts
        src -> do
            res <- getData (fromAttrib "type" t) src
            case res of
-                Left dataUri -> do
+                AlreadyDataURI dataUri -> do
                   rest <- convertTags ts
                   return $ TagOpen "link"
                      (("href",dataUri) : [(x,y) | (x,y) <- as, x /= "href"]) :
                      rest
-                Right (mime, bs)
+                Fetched (mime, bs)
                   | "text/css" `T.isPrefixOf` mime
                     && T.null (fromAttrib "media" t)
                     && not ("</" `B.isInfixOf` bs) -> do
@@ -113,6 +116,9 @@ convertTags (t@(TagOpen "link" as):ts) =
                       return $ TagOpen "link"
                        (("href",makeDataURI (mime, bs)) :
                          [(x,y) | (x,y) <- as, x /= "href"]) : rest
+                CouldNotFetch _ -> do
+                      rest <- convertTags ts
+                      return $ t:rest
 convertTags (t@(TagOpen tagname as):ts)
   | any (isSourceAttribute tagname) as
      = do
@@ -122,9 +128,13 @@ convertTags (t@(TagOpen tagname as):ts)
   where processAttribute (x,y) =
            if isSourceAttribute tagname (x,y)
               then do
-                enc <- getDataURI (fromAttrib "type" t) y
-                return (x, enc)
+                res <- getData (fromAttrib "type" t) y
+                case res of
+                  AlreadyDataURI enc -> return (x, enc)
+                  Fetched (mt,bs) -> return (x, makeDataURI (mt,bs))
+                  CouldNotFetch _ -> return (x, y)
               else return (x,y)
+
 convertTags (t:ts) = (t:) <$> convertTags ts
 
 cssURLs :: PandocMonad m
@@ -213,8 +223,8 @@ handleCSSUrl d (url, fallback) =
       u ->  do let url' = if isURI (T.pack u) then T.pack u else T.pack (d </> u)
                res <- lift $ getData "" url'
                case res of
-                    Left uri -> return $ Left (fromText $ "url(" <> uri <> ")")
-                    Right (mt', raw) -> do
+                    AlreadyDataURI uri -> return $ Left (fromText $ "url(" <> uri <> ")")
+                    Fetched (mt', raw) -> do
                       -- note that the downloaded CSS may
                       -- itself contain url(...).
                       (mt, b) <- if "text/css" `T.isPrefixOf` mt'
@@ -223,20 +233,22 @@ handleCSSUrl d (url, fallback) =
                                     then ("text/css",) <$> cssURLs d raw
                                     else return (mt', raw)
                       return $ Right (mt, b)
+                    CouldNotFetch _ -> return $ Left fallback
 
-getDataURI :: PandocMonad m => MimeType -> T.Text -> m T.Text
-getDataURI mimetype src = do
-  res <- getData mimetype src
-  case res of
-       Left uri -> return uri
-       Right x  -> return $ makeDataURI x
+data GetDataResult =
+    AlreadyDataURI T.Text
+  | CouldNotFetch PandocError
+  | Fetched (MimeType, ByteString)
+  deriving (Show)
 
 getData :: PandocMonad m
         => MimeType -> T.Text
-        -> m (Either T.Text (MimeType, ByteString))
+        -> m GetDataResult
 getData mimetype src
-  | "data:" `T.isPrefixOf` src = return $ Left src -- already data: uri
-  | otherwise = do
+  | "data:" `T.isPrefixOf` src = return $ AlreadyDataURI src -- already data: uri
+  | otherwise = catchError fetcher handler
+ where
+   fetcher = do
       let ext = T.toLower $ T.pack $ takeExtension $ T.unpack src
       (raw, respMime) <- fetchItem src
       let raw' = if ext `elem` [".gz", ".svgz"]
@@ -254,7 +266,16 @@ getData mimetype src
                   setInputFiles oldInputs
                   return res
                else return raw'
-      return $ Right (mime, result)
+      return $ Fetched (mime, result)
+   handler e = case e of
+                 PandocResourceNotFound r -> do
+                   report $ CouldNotFetchResource r ""
+                   return $ CouldNotFetch e
+                 PandocHttpError u er -> do
+                   report $ CouldNotFetchResource u (tshow er)
+                   return $ CouldNotFetch e
+                 _ -> throwError e
+
 
 
 
