@@ -27,7 +27,7 @@ import Data.Char (isSpace, isLetter)
 import Data.List (intercalate, isPrefixOf, isSuffixOf)
 import Data.String (fromString)
 import qualified Data.Map as M
-import Data.Maybe (fromMaybe, isNothing, mapMaybe, maybeToList)
+import Data.Maybe (fromMaybe, isNothing, mapMaybe, maybeToList, isJust)
 import qualified Data.Set as Set
 import qualified Data.Text as T
 import Data.Text (Text)
@@ -930,10 +930,12 @@ blockToOpenXML' _ HorizontalRule = do
     $ mknode "v:rect" [("style","width:0;height:1.5pt"),
                        ("o:hralign","center"),
                        ("o:hrstd","t"),("o:hr","t")] () ]
-blockToOpenXML' opts (Table attr caption colspecs thead tbodies tfoot) =
-  tableToOpenXML opts
+blockToOpenXML' opts (Table attr caption colspecs thead tbodies tfoot) = do
+  content <- tableToOpenXML opts
                  (blocksToOpenXML opts)
                  (Grid.toTable attr caption colspecs thead tbodies tfoot)
+  let (tableId, _, _) = attr
+  wrapBookmark tableId content
 blockToOpenXML' opts el
   | BulletList lst <- el = addOpenXMLList BulletMarker lst
   | OrderedList (start, numstyle, numdelim) lst <- el
@@ -942,7 +944,10 @@ blockToOpenXML' opts el
     addOpenXMLList marker lst = do
       addList marker
       numid  <- getNumId
-      l <- asList $ concat `fmap` mapM (listItemToOpenXML opts numid) lst
+      exampleid <- case marker of
+                        NumberMarker Example _ _ -> gets stExampleId
+                        _ -> return Nothing
+      l <- asList $ concat `fmap` mapM (listItemToOpenXML opts $ fromMaybe numid exampleid) lst
       setFirstPara
       return l
 blockToOpenXML' opts (DefinitionList items) = do
@@ -963,7 +968,16 @@ definitionListItemToOpenXML opts (term,defs) = do
 addList :: (PandocMonad m) => ListMarker -> WS m ()
 addList marker = do
   lists <- gets stLists
-  modify $ \st -> st{ stLists = lists ++ [marker] }
+  lastExampleId <- gets stExampleId
+  modify $ \st -> st{ stLists = lists ++ case marker of
+                                         -- Use only first occurrence of Example for list declaration to avoid overhead
+                                         NumberMarker Example _ _ | isJust lastExampleId -> []
+                                         _ -> [marker]
+                    , stExampleId = case marker of
+                                         -- Reuse the same identifier for all other occurrences of Example
+                                         NumberMarker Example _ _ -> lastExampleId <|> Just (baseListId + length lists)
+                                         _ -> lastExampleId
+                  }
 
 listItemToOpenXML :: (PandocMonad m)
                   => WriterOptions
