@@ -23,7 +23,7 @@ module Text.Pandoc.Writers.Markdown (
 import Control.Monad.Reader
 import Control.Monad.State.Strict
 import Data.Default
-import Data.List (intersperse, sortOn, transpose)
+import Data.List (intersperse, sortOn)
 import Data.List.NonEmpty (nonEmpty, NonEmpty(..))
 import qualified Data.Map as M
 import Data.Maybe (fromMaybe, mapMaybe, isNothing)
@@ -43,10 +43,11 @@ import Text.Pandoc.Templates (renderTemplate)
 import Text.DocTemplates (Val(..), Context(..), FromContext(..))
 import Text.Pandoc.Walk
 import Text.Pandoc.Writers.HTML (writeHtml5String)
-import Text.Pandoc.Writers.Markdown.Inline (inlineListToMarkdown, 
-                                            linkAttributes, 
-                                            attrsToMarkdown, 
+import Text.Pandoc.Writers.Markdown.Inline (inlineListToMarkdown,
+                                            linkAttributes,
+                                            attrsToMarkdown,
                                             attrsToMarkua)
+import Text.Pandoc.Writers.Markdown.Table (pipeTable, pandocTable)
 import Text.Pandoc.Writers.Markdown.Types (MarkdownVariant(..),
                                            WriterState(..),
                                            WriterEnv(..),
@@ -356,7 +357,7 @@ blockToMarkdown' opts (Div attrs ils) = do
   variant <- asks envVariant
   return $
      case () of
-         _ | variant == Markua -> 
+         _ | variant == Markua ->
                    case () of
                         () | "blurb" `elem` classes' -> prefixed "B> " contents <> blankline
                            | "aside" `elem` classes' -> prefixed "A> " contents <> blankline
@@ -470,13 +471,15 @@ blockToMarkdown' opts (Header level attr inlines) = do
           else return empty
   variant <- asks envVariant
   -- we calculate the id that would be used by auto_identifiers
+  -- or gfm_auto_identifiers
   -- so we know whether to print an explicit identifier
   ids <- gets stIds
   let autoId = uniqueIdent (writerExtensions opts) inlines ids
   modify $ \st -> st{ stIds = Set.insert autoId ids }
   let attr' = case attr of
                    ("",[],[]) -> empty
-                   (id',[],[]) | isEnabled Ext_auto_identifiers opts
+                   (id',[],[]) | (isEnabled Ext_auto_identifiers opts
+                                  || isEnabled Ext_gfm_auto_identifiers opts)
                                  && id' == autoId -> empty
                    (id',_,_)   | isEnabled Ext_mmd_header_identifiers opts ->
                                     space <> brackets (literal id')
@@ -515,7 +518,7 @@ blockToMarkdown' opts (Header level attr inlines) = do
             -- ghc interprets '#' characters in column 1 as linenum specifiers.
             _ | variant == PlainText || isEnabled Ext_literate_haskell opts ->
                 contents <> blankline
-            _ | variant == Markua -> attr' <> cr <> literal (T.replicate level "#") 
+            _ | variant == Markua -> attr' <> cr <> literal (T.replicate level "#")
                                         <> space <> contents <> blankline
             _ -> literal (T.replicate level "#") <> space <> contents <> attr' <> blankline
 
@@ -533,7 +536,7 @@ blockToMarkdown' opts (CodeBlock attribs str) = do
           backticks <> attrs <> cr <> literal str <> cr <> backticks <> blankline
            | isEnabled Ext_fenced_code_blocks opts ->
           tildes <> attrs <> cr <> literal str <> cr <> tildes <> blankline
-     _ | variant == Markua -> blankline <> attrsToMarkua attribs <> cr <> backticks <> cr <> 
+     _ | variant == Markua -> blankline <> attrsToMarkua attribs <> cr <> backticks <> cr <>
                                 literal str <> cr <> backticks <> cr <> blankline
        | otherwise -> nest (writerTabStop opts) (literal str) <> blankline
    where
@@ -626,7 +629,7 @@ blockToMarkdown' opts t@(Table _ blkCapt specs thead tbody tfoot) = do
   return $ nst (tbl $$ caption'') $$ blankline
 blockToMarkdown' opts (BulletList items) = do
   contents <- inList $ mapM (bulletListItemToMarkdown opts) items
-  return $ (if isTightList items then vcat else vsep) 
+  return $ (if isTightList items then vcat else vsep)
                   contents <> blankline
 blockToMarkdown' opts (OrderedList (start,sty,delim) items) = do
   variant <- asks envVariant
@@ -635,7 +638,7 @@ blockToMarkdown' opts (OrderedList (start,sty,delim) items) = do
                   else 1
   let sty'   = if isEnabled Ext_fancy_lists opts then sty else DefaultStyle
   let delim' | isEnabled Ext_fancy_lists opts =
-               case variant of 
+               case variant of
                    -- Markua supports 'fancy' enumerators, but no TwoParens
                    Markua -> if delim == TwoParens then OneParen else delim
                    _ -> delim
@@ -668,102 +671,6 @@ addMarkdownAttribute s =
                                  x /= "markdown"]
        _ -> s
 
-pipeTable :: PandocMonad m
-          => WriterOptions
-          -> Bool -> [Alignment] -> [Double] -> [Doc Text] -> [[Doc Text]]
-          -> MD m (Doc Text)
-pipeTable opts headless aligns widths rawHeaders rawRows = do
-  let sp = literal " "
-  let blockFor AlignLeft   x y = lblock (x + 2) (sp <> y) <> lblock 0 empty
-      blockFor AlignCenter x y = cblock (x + 2) (sp <> y <> sp) <> lblock 0 empty
-      blockFor AlignRight  x y = rblock (x + 2) (y <> sp) <> lblock 0 empty
-      blockFor _           x y = lblock (x + 2) (sp <> y) <> lblock 0 empty
-  let contentWidths = map (max 3 . maybe 3 maximum . nonEmpty . map offset) $
-                       transpose (rawHeaders : rawRows)
-  let colwidth = writerColumns opts
-  let numcols = length contentWidths
-  let maxwidth = sum contentWidths
-  variant <- asks envVariant
-  let pipeWidths = if variant == Markdown &&
-                      not (all (== 0) widths) &&
-                      maxwidth + (numcols + 1) > colwidth
-                      then map
-                            (floor . (* fromIntegral (colwidth - (numcols +1))))
-                            widths
-                      else contentWidths
-  let torow cs = nowrap $ literal "|" <>
-                    hcat (intersperse (literal "|") $
-                          zipWith3 blockFor aligns contentWidths (map chomp cs))
-                    <> literal "|"
-  let toborder a w = literal $ case a of
-                          AlignLeft    -> ":" <> T.replicate (w + 1) "-"
-                          AlignCenter  -> ":" <> T.replicate w "-" <> ":"
-                          AlignRight   -> T.replicate (w + 1) "-" <> ":"
-                          AlignDefault -> T.replicate (w + 2) "-"
-  -- note:  pipe tables can't completely lack a
-  -- header; for a headerless table, we need a header of empty cells.
-  -- see jgm/pandoc#1996.
-  let header = if headless
-                  then torow (replicate (length aligns) empty)
-                  else torow rawHeaders
-  let border = nowrap $ literal "|" <> hcat (intersperse (literal "|") $
-                        zipWith toborder aligns pipeWidths) <> literal "|"
-  let body   = vcat $ map torow rawRows
-  return $ header $$ border $$ body
-
-pandocTable :: PandocMonad m
-            => WriterOptions -> Bool -> Bool -> [Alignment] -> [Double]
-            -> [Doc Text] -> [[Doc Text]] -> MD m (Doc Text)
-pandocTable opts multiline headless aligns widths rawHeaders rawRows = do
-  let isSimple = all (==0) widths
-  let alignHeader alignment = case alignment of
-                                AlignLeft    -> lblock
-                                AlignCenter  -> cblock
-                                AlignRight   -> rblock
-                                AlignDefault -> lblock
-  -- Number of characters per column necessary to output every cell
-  -- without requiring a line break.
-  -- The @+2@ is needed for specifying the alignment.
-  let numChars    = (+ 2) . maybe 0 maximum . nonEmpty . map offset
-  -- Number of characters per column necessary to output every cell
-  -- without requiring a line break *inside a word*.
-  -- The @+2@ is needed for specifying the alignment.
-  let minNumChars = (+ 2) . maybe 0 maximum . nonEmpty . map minOffset
-  let columns = transpose (rawHeaders : rawRows)
-  -- minimal column width without wrapping a single word
-  let relWidth w col =
-         max (floor $ fromIntegral (writerColumns opts - 1) * w)
-             (if writerWrapText opts == WrapAuto
-                 then minNumChars col
-                 else numChars col)
-  let widthsInChars
-        | isSimple  = map numChars columns
-        | otherwise = zipWith relWidth widths columns
-  let makeRow = hcat . intersperse (lblock 1 (literal " ")) .
-                   zipWith3 alignHeader aligns widthsInChars
-  let rows' = map makeRow rawRows
-  let head' = makeRow rawHeaders
-  let underline = mconcat $ intersperse (literal " ") $
-                  map (\width -> literal (T.replicate width "-")) widthsInChars
-  let border
-        | multiline = literal (T.replicate (sum widthsInChars +
-                        length widthsInChars - 1) "-")
-        | headless  = underline
-        | otherwise = empty
-  let head'' = if headless
-                  then empty
-                  else border <> cr <> head'
-  let body = if multiline
-                then vsep rows' $$
-                     if length rows' < 2
-                        then blankline -- #4578
-                        else empty
-                else vcat rows'
-  let bottom = if headless
-                  then underline
-                  else border
-  return $ head'' $$ underline $$ body $$ bottom
-
 itemEndsWithTightList :: [Block] -> Bool
 itemEndsWithTightList bs =
   case bs of
