@@ -146,7 +146,7 @@ blockToOrg (Header level attr inlines) = do
   let headerStr = text $ if level > 999 then " " else replicate level '*'
   let drawerStr = if attr == nullAttr
                   then empty
-                  else cr <> nest (level + 1) (propertiesDrawer attr)
+                  else cr <> propertiesDrawer attr
   return $ headerStr <> " " <> contents <> drawerStr <> cr
 blockToOrg (CodeBlock (_,classes,kvs) str) = do
   let startnum = maybe "" (\x -> " " <> trimr x) $ lookup "startFrom" kvs
@@ -163,7 +163,7 @@ blockToOrg (CodeBlock (_,classes,kvs) str) = do
 blockToOrg (BlockQuote blocks) = do
   contents <- blockListToOrg blocks
   return $ blankline $$ "#+begin_quote" $$
-           nest 2 contents $$ "#+end_quote" $$ blankline
+           contents $$ "#+end_quote" $$ blankline
 blockToOrg (Table _ blkCapt specs thead tbody tfoot) =  do
   let (caption', _, _, headers, rows) = toLegacyTable blkCapt specs thead tbody tfoot
   caption'' <- inlineListToOrg caption'
