@@ -1,8 +1,7 @@
 {-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE ViewPatterns      #-}
 {- |
    Module      : Text.Pandoc.Writers.Textile
-   Copyright   : Copyright (C) 2010-2020 John MacFarlane
+   Copyright   : Copyright (C) 2010-2022 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley.edu>
@@ -111,8 +110,7 @@ blockToTextile opts (Div attr bs) = do
 blockToTextile opts (Plain inlines) =
   inlineListToTextile opts inlines
 
--- title beginning with fig: indicates that the image is a figure
-blockToTextile opts (Para [Image attr txt (src,T.stripPrefix "fig:" -> Just tit)]) = do
+blockToTextile opts (SimpleFigure attr txt (src, tit)) = do
   capt <- blockToTextile opts (Para txt)
   im <- inlineToTextile opts (Image attr txt (src,tit))
   return $ im <> "\n" <> capt
@@ -168,44 +166,44 @@ blockToTextile opts (BlockQuote blocks) = do
   contents <- blockListToTextile opts blocks
   return $ "<blockquote>\n\n" <> contents <> "\n</blockquote>\n"
 
-blockToTextile opts (Table [] aligns widths headers rows') |
-         all (==0) widths = do
-  hs <- mapM (liftM (("_. " <>) . stripTrailingNewlines) . blockListToTextile opts) headers
-  let cellsToRow cells = "|" <> T.intercalate "|" cells <> "|"
-  let header = if all null headers then "" else cellsToRow hs <> "\n"
-  let blocksToCell (align, bs) = do
-        contents <- stripTrailingNewlines <$> blockListToTextile opts bs
-        let alignMarker = case align of
-                               AlignLeft    -> "<. "
-                               AlignRight   -> ">. "
-                               AlignCenter  -> "=. "
-                               AlignDefault -> ""
-        return $ alignMarker <> contents
-  let rowToCells = mapM blocksToCell . zip aligns
-  bs <- mapM rowToCells rows'
-  let body = T.unlines $ map cellsToRow bs
-  return $ header <> body
-
-blockToTextile opts (Table capt aligns widths headers rows') = do
-  let alignStrings = map alignmentToText aligns
-  captionDoc <- if null capt
-                   then return ""
-                   else do
-                      c <- inlineListToTextile opts capt
-                      return $ "<caption>" <> c <> "</caption>\n"
-  let percent w = tshow (truncate (100*w) :: Integer) <> "%"
-  let coltags = if all (== 0.0) widths
-                   then ""
-                   else T.unlines $ map
-                         (\w -> "<col width=\"" <> percent w <> "\" />") widths
-  head' <- if all null headers
-              then return ""
-              else do
-                 hs <- tableRowToTextile opts alignStrings 0 headers
-                 return $ "<thead>\n" <> hs <> "\n</thead>\n"
-  body' <- zipWithM (tableRowToTextile opts alignStrings) [1..] rows'
-  return $ "<table>\n" <> captionDoc <> coltags <> head' <>
-            "<tbody>\n" <> T.unlines body' <> "</tbody>\n</table>\n"
+blockToTextile opts (Table _ blkCapt specs thead tbody tfoot)
+  = case toLegacyTable blkCapt specs thead tbody tfoot of
+      ([], aligns, widths, headers, rows') | all (==0) widths -> do
+        hs <- mapM (liftM (("_. " <>) . stripTrailingNewlines) . blockListToTextile opts) headers
+        let cellsToRow cells = "|" <> T.intercalate "|" cells <> "|"
+        let header = if all null headers then "" else cellsToRow hs <> "\n"
+        let blocksToCell (align, bs) = do
+              contents <- stripTrailingNewlines <$> blockListToTextile opts bs
+              let alignMarker = case align of
+                                     AlignLeft    -> "<. "
+                                     AlignRight   -> ">. "
+                                     AlignCenter  -> "=. "
+                                     AlignDefault -> ""
+              return $ alignMarker <> contents
+        let rowToCells = mapM blocksToCell . zip aligns
+        bs <- mapM rowToCells rows'
+        let body = T.unlines $ map cellsToRow bs
+        return $ header <> body
+      (capt, aligns, widths, headers, rows') -> do
+        let alignStrings = map alignmentToText aligns
+        captionDoc <- if null capt
+                         then return ""
+                         else do
+                            c <- inlineListToTextile opts capt
+                            return $ "<caption>" <> c <> "</caption>\n"
+        let percent w = tshow (truncate (100*w) :: Integer) <> "%"
+        let coltags = if all (== 0.0) widths
+                         then ""
+                         else T.unlines $ map
+                               (\w -> "<col width=\"" <> percent w <> "\" />") widths
+        head' <- if all null headers
+                    then return ""
+                    else do
+                       hs <- tableRowToTextile opts alignStrings 0 headers
+                       return $ "<thead>\n" <> hs <> "\n</thead>\n"
+        body' <- zipWithM (tableRowToTextile opts alignStrings) [1..] rows'
+        return $ "<table>\n" <> captionDoc <> coltags <> head' <>
+                  "<tbody>\n" <> T.unlines body' <> "</tbody>\n</table>\n"
 
 blockToTextile opts x@(BulletList items) = do
   oldUseTags <- gets stUseTags
@@ -386,6 +384,12 @@ inlineToTextile opts (Emph lst) = do
               then "<em>" <> contents <> "</em>"
               else "_" <> contents <> "_"
 
+inlineToTextile opts (Underline lst) = do
+  contents <- inlineListToTextile opts lst
+  return $ if '+' `elemText` contents
+              then "<u>" <> contents <> "</u>"
+              else "+" <> contents <> "+"
+
 inlineToTextile opts (Strong lst) = do
   contents <- inlineListToTextile opts lst
   return $ if '*' `elemText` contents
