@@ -370,6 +370,26 @@ tests = [ testGroup "document"
             "docx/drop_cap.docx"
             "docx/drop_cap.native"
           ]
+        , testGroup "citations"
+          [ testCompare
+            "zotero with -citations"
+            "docx/zotero_citations.docx"
+            "docx/zotero_citations_minus.native"
+          , testCompareWithOpts def{readerExtensions =
+                  extensionsFromList [Ext_citations]}
+            "zotero with +citations"
+            "docx/zotero_citations.docx"
+            "docx/zotero_citations_plus.native"
+          , testCompare
+            "mendeley with -citations"
+            "docx/mendeley_citations.docx"
+            "docx/mendeley_citations_minus.native"
+          , testCompareWithOpts def{readerExtensions =
+                  extensionsFromList [Ext_citations]}
+            "mendeley with +citations"
+            "docx/mendeley_citations.docx"
+            "docx/mendeley_citations_plus.native"
+          ]
         , testGroup "track changes"
           [ testCompare
             "insertion (default)"
