@@ -1,5 +1,6 @@
 {-# LANGUAGE OverloadedStrings   #-}
 {-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE TupleSections #-}
 {- |
    Module      : Tests.Readers.DokuWiki
    Copyright   : © 2018-2020 Alexander Krotov
@@ -16,6 +17,7 @@ module Tests.Readers.DokuWiki (tests) where
 import Data.Text (Text)
 import qualified Data.Text as T
 import Test.Tasty
+import Test.Tasty.HUnit (HasCallStack)
 import Tests.Helpers
 import Text.Pandoc
 import Text.Pandoc.Arbitrary ()
@@ -25,7 +27,7 @@ dokuwiki :: Text -> Pandoc
 dokuwiki = purely $ readDokuWiki def{ readerStandalone = True }
 
 infix 4 =:
-(=:) :: ToString c
+(=:) :: (ToString c, HasCallStack)
      => String -> (Text, c) -> TestTree
 (=:) = test dokuwiki
 
@@ -224,12 +226,12 @@ tests = [ testGroup "inlines"
           , "Ordered list" =:
             T.unlines [ "  - The same list but ordered"
                       , "  - Another item"
-                      , "    - Just use indention for deeper levels"
+                      , "    - Just use indentation for deeper levels"
                       , "  - That's it"
                       ] =?>
             orderedList [ plain "The same list but ordered"
                         , plain "Another item" <>
-                          orderedList [ plain "Just use indention for deeper levels" ]
+                          orderedList [ plain "Just use indentation for deeper levels" ]
                         , plain "That's it"
                         ]
           , "Multiline list items" =: -- https://www.dokuwiki.org/faq:lists
@@ -300,6 +302,17 @@ tests = [ testGroup "inlines"
                     , "| bat | baz |"
                     ] =?>
           simpleTable [plain "foo", plain "bar"] [[plain "bat", plain "baz"]]
+        , "Table with alignment" =:
+          T.unlines [ "^ 0  ^  1  ^  2 ^ 3 ^"
+                    , "| a  | b   | c  |d  |"
+                    ] =?>
+          table emptyCaption
+                (map (, ColWidthDefault) [AlignLeft, AlignCenter, AlignRight, AlignDefault])
+                (TableHead nullAttr
+                          [Row nullAttr . map (simpleCell . plain) $ ["0", "1", "2", "3"]])
+                [TableBody nullAttr 0 []
+                          [Row nullAttr . map (simpleCell . plain) $ ["a", "b", "c", "d"]]]
+                (TableFoot nullAttr [])
         , "Table with colspan" =:
           T.unlines [ "^ 0,0 ^ 0,1 ^ 0,2 ^"
                     , "| 1,0 | 1,1 ||"
