@@ -15,6 +15,7 @@ module Tests.Readers.Markdown (tests) where
 import Data.Text (Text, unpack)
 import qualified Data.Text as T
 import Test.Tasty
+import Test.Tasty.HUnit (HasCallStack)
 import Tests.Helpers
 import Text.Pandoc
 import Text.Pandoc.Arbitrary ()
@@ -40,7 +41,7 @@ markdownMMD :: Text -> Pandoc
 markdownMMD = purely $ readMarkdown def {
                  readerExtensions = multimarkdownExtensions }
 infix 4 =:
-(=:) :: ToString c
+(=:) :: (ToString c, HasCallStack)
      => String -> (Text, c) -> TestTree
 (=:) = test markdown
 
@@ -371,13 +372,13 @@ tests = [ testGroup "inline code"
           , test markdownMMD "normal superscript"
             ("x^3^"
             =?> para ("x" <> superscript "3"))
-          , test markdownMMD "short subscript delimeted by space"
+          , test markdownMMD "short subscript delimited by space"
             ("O~2 is dangerous"
             =?> para ("O" <> subscript "2" <> space <> "is dangerous"))
-          , test markdownMMD "short subscript delimeted by newline"
+          , test markdownMMD "short subscript delimited by newline"
             ("O~2\n"
             =?> para ("O" <> subscript "2"))
-          , test markdownMMD "short subscript delimeted by EOF"
+          , test markdownMMD "short subscript delimited by EOF"
             ("O~2"
             =?> para ("O" <> subscript "2"))
           , test markdownMMD "short subscript delimited by punctuation"
@@ -389,13 +390,13 @@ tests = [ testGroup "inline code"
           , test markdownMMD "no nesting in short subscripts"
             ("y~*2*"
             =?> para ("y~" <> emph "2"))
-          , test markdownMMD "short superscript delimeted by space"
+          , test markdownMMD "short superscript delimited by space"
             ("x^2 = y"
             =?> para ("x" <> superscript "2" <> space <> "= y"))
-          , test markdownMMD "short superscript delimeted by newline"
+          , test markdownMMD "short superscript delimited by newline"
             ("x^2\n"
             =?> para ("x" <> superscript "2"))
-          , test markdownMMD "short superscript delimeted by ExF"
+          , test markdownMMD "short superscript delimited by ExF"
             ("x^2"
             =?> para ("x" <> superscript "2"))
           , test markdownMMD "short superscript delimited by punctuation"
