@@ -1,8 +1,7 @@
-{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE OverloadedStrings #-}
 {- |
    Module      : Tests.Readers.Org.Block.Figure
-   Copyright   : © 2014-2020 Albert Krewinkel
+   Copyright   : © 2014-2022 Albert Krewinkel
    License     : GNU GPL, version 2 or above
 
    Maintainer  : Albert Krewinkel <albert@zeitkraut.de>
@@ -13,7 +12,6 @@ Test parsing of org figures.
 -}
 module Tests.Readers.Org.Block.Figure (tests) where
 
-import Prelude
 import Test.Tasty (TestTree)
 import Tests.Helpers ((=?>))
 import Tests.Readers.Org.Shared ((=:))
@@ -44,9 +42,9 @@ tests =
                   "Used as a metapher in evolutionary biology.")
 
   , "Figure with HTML attributes" =:
-      T.unlines [ "#+CAPTION: mah brain just explodid"
-                , "#+NAME: lambdacat"
-                , "#+ATTR_HTML: :style color: blue :role button"
+      T.unlines [ "#+caption: mah brain just explodid"
+                , "#+name: lambdacat"
+                , "#+attr_html: :style color: blue :role button"
                 , "[[file:lambdacat.jpg]]"
                 ] =?>
       let kv = [("style", "color: blue"), ("role", "button")]
@@ -55,22 +53,22 @@ tests =
       in para (imageWith (mempty, mempty, kv) "lambdacat.jpg" name caption)
 
   , "LaTeX attributes are ignored" =:
-      T.unlines [ "#+CAPTION: Attribute after caption"
-                , "#+ATTR_LATEX: :float nil"
+      T.unlines [ "#+caption: Attribute after caption"
+                , "#+attr_latex: :float nil"
                 , "[[file:test.png]]"
                 ] =?>
       para (image "test.png" "fig:" "Attribute after caption")
 
   , "Labelled figure" =:
-      T.unlines [ "#+CAPTION: My figure"
-                , "#+LABEL: fig:myfig"
+      T.unlines [ "#+caption: My figure"
+                , "#+label: fig:myfig"
                 , "[[file:blub.png]]"
                 ] =?>
       let attr = ("fig:myfig", mempty, mempty)
       in para (imageWith attr "blub.png" "fig:" "My figure")
 
   , "Figure with empty caption" =:
-      T.unlines [ "#+CAPTION:"
+      T.unlines [ "#+caption:"
                 , "[[file:guess.jpg]]"
                 ] =?>
       para (image "guess.jpg" "fig:" "")
