@@ -250,7 +250,7 @@ tests =
                     , "\\end{equation}"
                     ] =?>
           para (str "\\begin{equation}" <> softbreak <>
-                str "f(x) = x^2" <> softbreak <>
+                text "f(x) = x^2" <> softbreak <>
                 str "\\end{equation}")
         ]
       ]
