@@ -1,7 +1,5 @@
-{-# LANGUAGE NoImplicitPrelude #-}
 module Tests.Writers.Docx (tests) where
 
-import Prelude
 import Text.Pandoc
 import Test.Tasty
 import Tests.Writers.OOXML
@@ -33,7 +31,8 @@ tests = [ testGroup "inlines"
             "docx/golden/links.docx"
           , docxTest
             "inline image"
-            def
+            def{ writerExtensions =
+                   enableExtension Ext_native_numbering (writerExtensions def) }
             "docx/image_writer_test.native"
             "docx/golden/image.docx"
           , docxTest
@@ -83,6 +82,16 @@ tests = [ testGroup "inlines"
             def
             "docx/lists_restarting.native"
             "docx/golden/lists_restarting.docx"
+          , docxTest
+            "lists with multiple initial list levels"
+            def
+            "docx/lists_multiple_initial.native"
+            "docx/golden/lists_multiple_initial.docx"
+          , docxTest
+            "lists with div bullets"
+            def
+            "docx/lists_div_bullets.native"
+            "docx/golden/lists_div_bullets.docx"
           , docxTest
             "definition lists"
             def
@@ -108,6 +117,11 @@ tests = [ testGroup "inlines"
             def
             "docx/tables.native"
             "docx/golden/tables.docx"
+          , docxTest
+            "tables without explicit column widths"
+            def
+            "docx/tables-default-widths.native"
+            "docx/golden/tables-default-widths.docx"
           , docxTest
             "tables with lists in cells"
             def
@@ -118,11 +132,26 @@ tests = [ testGroup "inlines"
             def
             "docx/table_one_row.native"
             "docx/golden/table_one_row.docx"
+          , docxTest
+            "tables separated with RawBlock"
+            def
+            "docx/tables_separated_with_rawblock.native"
+            "docx/golden/tables_separated_with_rawblock.docx"
           , docxTest
             "code block"
             def
             "docx/codeblock.native"
             "docx/golden/codeblock.docx"
+          , docxTest
+            "raw OOXML blocks"
+            def
+            "docx/raw-blocks.native"
+            "docx/golden/raw-blocks.docx"
+          , docxTest
+            "raw bookmark markers"
+            def
+            "docx/raw-bookmarks.native"
+            "docx/golden/raw-bookmarks.docx"
           ]
         , testGroup "track changes"
           [ docxTest
@@ -145,6 +174,11 @@ tests = [ testGroup "inlines"
             def
             "docx/comments.native"
             "docx/golden/comments.docx"
+          , docxTest
+            "scrubbed metadata"
+            def
+            "docx/track_changes_scrubbed_metadata.native"
+            "docx/golden/track_changes_scrubbed_metadata.docx"
           ]
         , testGroup "custom styles"
           [ docxTest "custom styles without reference.docx"
